@@ -19,7 +19,10 @@ try:
 except ImportError:
     boto = None  # type: ignore[assignment]
 
-from galaxy.exceptions import ObjectInvalid, ObjectNotFound
+from galaxy.exceptions import (
+    ObjectInvalid,
+    ObjectNotFound,
+)
 from galaxy.util import (
     directory_hash_id,
     string_as_bool,
@@ -29,74 +32,79 @@ from galaxy.util import (
 )
 from galaxy.util.path import safe_relpath
 from galaxy.util.sleeper import Sleeper
+from . import (
+    ConcreteObjectStore,
+    convert_bytes,
+)
 from .s3_multipart_upload import multipart_upload
-from ..objectstore import ConcreteObjectStore, convert_bytes
 
-NO_BOTO_ERROR_MESSAGE = ("S3/Swift object store configured, but no boto dependency available."
-                         "Please install and properly configure boto or modify object store configuration.")
+NO_BOTO_ERROR_MESSAGE = (
+    "S3/Swift object store configured, but no boto dependency available."
+    "Please install and properly configure boto or modify object store configuration."
+)
 
 log = logging.getLogger(__name__)
-logging.getLogger('boto').setLevel(logging.INFO)  # Otherwise boto is quite noisy
+logging.getLogger("boto").setLevel(logging.INFO)  # Otherwise boto is quite noisy
 
 
 def parse_config_xml(config_xml):
     try:
-        a_xml = config_xml.findall('auth')[0]
-        access_key = a_xml.get('access_key')
-        secret_key = a_xml.get('secret_key')
+        a_xml = config_xml.findall("auth")[0]
+        access_key = a_xml.get("access_key")
+        secret_key = a_xml.get("secret_key")
 
-        b_xml = config_xml.findall('bucket')[0]
-        bucket_name = b_xml.get('name')
-        use_rr = string_as_bool(b_xml.get('use_reduced_redundancy', "False"))
-        max_chunk_size = int(b_xml.get('max_chunk_size', 250))
+        b_xml = config_xml.findall("bucket")[0]
+        bucket_name = b_xml.get("name")
+        use_rr = string_as_bool(b_xml.get("use_reduced_redundancy", "False"))
+        max_chunk_size = int(b_xml.get("max_chunk_size", 250))
 
-        cn_xml = config_xml.findall('connection')
+        cn_xml = config_xml.findall("connection")
         if not cn_xml:
             cn_xml = {}
         else:
             cn_xml = cn_xml[0]
 
-        host = cn_xml.get('host', None)
-        port = int(cn_xml.get('port', 6000))
-        multipart = string_as_bool(cn_xml.get('multipart', 'True'))
-        is_secure = string_as_bool(cn_xml.get('is_secure', 'True'))
-        conn_path = cn_xml.get('conn_path', '/')
+        host = cn_xml.get("host", None)
+        port = int(cn_xml.get("port", 6000))
+        multipart = string_as_bool(cn_xml.get("multipart", "True"))
+        is_secure = string_as_bool(cn_xml.get("is_secure", "True"))
+        conn_path = cn_xml.get("conn_path", "/")
 
-        c_xml = config_xml.findall('cache')[0]
-        cache_size = float(c_xml.get('size', -1))
+        c_xml = config_xml.findall("cache")[0]
+        cache_size = float(c_xml.get("size", -1))
 
-        staging_path = c_xml.get('path', None)
+        staging_path = c_xml.get("path", None)
 
-        tag, attrs = 'extra_dir', ('type', 'path')
+        tag, attrs = "extra_dir", ("type", "path")
         extra_dirs = config_xml.findall(tag)
         if not extra_dirs:
-            msg = f'No {tag} element in XML tree'
+            msg = f"No {tag} element in XML tree"
             log.error(msg)
             raise Exception(msg)
         extra_dirs = [{k: e.get(k) for k in attrs} for e in extra_dirs]
 
         return {
-            'auth': {
-                'access_key': access_key,
-                'secret_key': secret_key,
+            "auth": {
+                "access_key": access_key,
+                "secret_key": secret_key,
             },
-            'bucket': {
-                'name': bucket_name,
-                'use_reduced_redundancy': use_rr,
-                'max_chunk_size': max_chunk_size,
+            "bucket": {
+                "name": bucket_name,
+                "use_reduced_redundancy": use_rr,
+                "max_chunk_size": max_chunk_size,
             },
-            'connection': {
-                'host': host,
-                'port': port,
-                'multipart': multipart,
-                'is_secure': is_secure,
-                'conn_path': conn_path,
+            "connection": {
+                "host": host,
+                "port": port,
+                "multipart": multipart,
+                "is_secure": is_secure,
+                "conn_path": conn_path,
             },
-            'cache': {
-                'size': cache_size,
-                'path': staging_path,
+            "cache": {
+                "size": cache_size,
+                "path": staging_path,
             },
-            'extra_dirs': extra_dirs,
+            "extra_dirs": extra_dirs,
         }
     except Exception:
         # Toss it back up after logging, we can't continue loading at this point.
@@ -105,29 +113,28 @@ def parse_config_xml(config_xml):
 
 
 class CloudConfigMixin:
-
     def _config_to_dict(self):
         return {
-            'auth': {
-                'access_key': self.access_key,
-                'secret_key': self.secret_key,
+            "auth": {
+                "access_key": self.access_key,
+                "secret_key": self.secret_key,
             },
-            'bucket': {
-                'name': self.bucket,
-                'use_reduced_redundancy': self.use_rr,
+            "bucket": {
+                "name": self.bucket,
+                "use_reduced_redundancy": self.use_rr,
             },
-            'connection': {
-                'host': self.host,
-                'port': self.port,
-                'multipart': self.multipart,
-                'is_secure': self.is_secure,
-                'conn_path': self.conn_path,
+            "connection": {
+                "host": self.host,
+                "port": self.port,
+                "multipart": self.multipart,
+                "is_secure": self.is_secure,
+                "conn_path": self.conn_path,
             },
-            'cache': {
-                'size': self.cache_size,
-                'path': self.staging_path,
+            "cache": {
+                "size": self.cache_size,
+                "path": self.staging_path,
             },
-            'enable_cache_monitor': False,
+            "enable_cache_monitor": False,
         }
 
 
@@ -137,37 +144,37 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
     cache exists that is used as an intermediate location for files between
     Galaxy and S3.
     """
-    store_type = 's3'
+
+    store_type = "s3"
 
     def __init__(self, config, config_dict):
         super().__init__(config, config_dict)
 
         self.transfer_progress = 0
 
-        auth_dict = config_dict['auth']
-        bucket_dict = config_dict['bucket']
-        connection_dict = config_dict.get('connection', {})
-        cache_dict = config_dict['cache']
-        self.enable_cache_monitor = config_dict.get('enable_cache_monitor', True)
+        auth_dict = config_dict["auth"]
+        bucket_dict = config_dict["bucket"]
+        connection_dict = config_dict.get("connection", {})
+        cache_dict = config_dict["cache"]
+        self.enable_cache_monitor = config_dict.get("enable_cache_monitor", True)
 
-        self.access_key = auth_dict.get('access_key')
-        self.secret_key = auth_dict.get('secret_key')
+        self.access_key = auth_dict.get("access_key")
+        self.secret_key = auth_dict.get("secret_key")
 
-        self.bucket = bucket_dict.get('name')
-        self.use_rr = bucket_dict.get('use_reduced_redundancy', False)
-        self.max_chunk_size = bucket_dict.get('max_chunk_size', 250)
+        self.bucket = bucket_dict.get("name")
+        self.use_rr = bucket_dict.get("use_reduced_redundancy", False)
+        self.max_chunk_size = bucket_dict.get("max_chunk_size", 250)
 
-        self.host = connection_dict.get('host', None)
-        self.port = connection_dict.get('port', 6000)
-        self.multipart = connection_dict.get('multipart', True)
-        self.is_secure = connection_dict.get('is_secure', True)
-        self.conn_path = connection_dict.get('conn_path', '/')
+        self.host = connection_dict.get("host", None)
+        self.port = connection_dict.get("port", 6000)
+        self.multipart = connection_dict.get("multipart", True)
+        self.is_secure = connection_dict.get("is_secure", True)
+        self.conn_path = connection_dict.get("conn_path", "/")
 
-        self.cache_size = cache_dict.get('size', -1)
-        self.staging_path = cache_dict.get('path') or self.config.object_store_cache_path
+        self.cache_size = cache_dict.get("size", -1)
+        self.staging_path = cache_dict.get("path") or self.config.object_store_cache_path
 
-        extra_dirs = {
-            e['type']: e['path'] for e in config_dict.get('extra_dirs', [])}
+        extra_dirs = {e["type"]: e["path"] for e in config_dict.get("extra_dirs", [])}
         self.extra_dirs.update(extra_dirs)
 
         self._initialize()
@@ -177,20 +184,22 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
             raise Exception(NO_BOTO_ERROR_MESSAGE)
 
         # for multipart upload
-        self.s3server = {'access_key': self.access_key,
-                         'secret_key': self.secret_key,
-                         'is_secure': self.is_secure,
-                         'max_chunk_size': self.max_chunk_size,
-                         'host': self.host,
-                         'port': self.port,
-                         'use_rr': self.use_rr,
-                         'conn_path': self.conn_path}
+        self.s3server = {
+            "access_key": self.access_key,
+            "secret_key": self.secret_key,
+            "is_secure": self.is_secure,
+            "max_chunk_size": self.max_chunk_size,
+            "host": self.host,
+            "port": self.port,
+            "use_rr": self.use_rr,
+            "conn_path": self.conn_path,
+        }
 
         self._configure_connection()
         self._bucket = self._get_bucket(self.bucket)
         self.start_cache_monitor()
         # Test if 'axel' is available for parallel download and pull the key into cache
-        if which('axel'):
+        if which("axel"):
             self.use_axel = True
         else:
             self.use_axel = False
@@ -244,8 +253,11 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
             # Initiate cleaning once within 10% of the defined cache size?
             cache_limit = self.cache_size * 0.9
             if total_size > cache_limit:
-                log.info("Initiating cache cleaning: current cache size: %s; clean until smaller than: %s",
-                         convert_bytes(total_size), convert_bytes(cache_limit))
+                log.info(
+                    "Initiating cache cleaning: current cache size: %s; clean until smaller than: %s",
+                    convert_bytes(total_size),
+                    convert_bytes(cache_limit),
+                )
                 # How much to delete? If simply deleting up to the cache-10% limit,
                 # is likely to be deleting frequently and may run the risk of hitting
                 # the limit - maybe delete additional #%?
@@ -255,7 +267,7 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
             self.sleeper.sleep(30)  # Test cache size every 30 seconds?
 
     def __clean_cache(self, file_list, delete_this_much):
-        """ Keep deleting files from the file_list until the size of the deleted
+        """Keep deleting files from the file_list until the size of the deleted
         files is greater than the value in delete_this_much parameter.
 
         :type file_list: list
@@ -288,8 +300,8 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
                 return
 
     def _get_bucket(self, bucket_name):
-        """ Sometimes a handle to a bucket is not established right away so try
-        it a few times. Raise error is connection is not established. """
+        """Sometimes a handle to a bucket is not established right away so try
+        it a few times. Raise error is connection is not established."""
         for i in range(5):
             try:
                 bucket = self.conn.get_bucket(bucket_name)
@@ -307,7 +319,7 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
         raise S3ResponseError
 
     def _fix_permissions(self, rel_path):
-        """ Set permissions on rel_path"""
+        """Set permissions on rel_path"""
         for basedir, _, files in os.walk(rel_path):
             umask_fix_perms(basedir, self.config.umask, 0o777, self.config.gid)
             for filename in files:
@@ -317,17 +329,27 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
                     continue
                 umask_fix_perms(path, self.config.umask, 0o666, self.config.gid)
 
-    def _construct_path(self, obj, base_dir=None, dir_only=None, extra_dir=None, extra_dir_at_root=False, alt_name=None, obj_dir=False, **kwargs):
+    def _construct_path(
+        self,
+        obj,
+        base_dir=None,
+        dir_only=None,
+        extra_dir=None,
+        extra_dir_at_root=False,
+        alt_name=None,
+        obj_dir=False,
+        **kwargs,
+    ):
         # extra_dir should never be constructed from provided data but just
         # make sure there are no shenannigans afoot
         if extra_dir and extra_dir != os.path.normpath(extra_dir):
-            log.warning('extra_dir is not normalized: %s', extra_dir)
+            log.warning("extra_dir is not normalized: %s", extra_dir)
             raise ObjectInvalid("The requested object is invalid")
         # ensure that any parent directory references in alt_name would not
         # result in a path not contained in the directory path constructed here
         if alt_name:
             if not safe_relpath(alt_name):
-                log.warning('alt_name would locate path outside dir: %s', alt_name)
+                log.warning("alt_name would locate path outside dir: %s", alt_name)
                 raise ObjectInvalid("The requested object is invalid")
             # alt_name can contain parent directory references, but S3 will not
             # follow them, so if they are valid we normalize them out
@@ -347,7 +369,7 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
             return os.path.join(base, rel_path)
 
         # S3 folders are marked by having trailing '/' so add it now
-        rel_path = f'{rel_path}/'
+        rel_path = f"{rel_path}/"
 
         if not dir_only:
             rel_path = os.path.join(rel_path, alt_name if alt_name else f"dataset_{self._get_object_id(obj)}.dat")
@@ -372,7 +394,7 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
         exists = False
         try:
             # A hackish way of testing if the rel_path is a folder vs a file
-            is_dir = rel_path[-1] == '/'
+            is_dir = rel_path[-1] == "/"
             if is_dir:
                 keyresult = self._bucket.get_all_keys(prefix=rel_path)
                 if len(keyresult) > 0:
@@ -388,7 +410,7 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
         return exists
 
     def _in_cache(self, rel_path):
-        """ Check if the given dataset is in the local cache and return True if so. """
+        """Check if the given dataset is in the local cache and return True if so."""
         # log.debug("------ Checking cache for rel_path %s" % rel_path)
         cache_path = self._get_cache_path(rel_path)
         return os.path.exists(cache_path)
@@ -433,16 +455,24 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
         try:
             log.debug("Pulling key '%s' into cache to %s", rel_path, self._get_cache_path(rel_path))
             key = self._bucket.get_key(rel_path)
+            if key is None:
+                message = f"Attempting to download an invalid key for path {rel_path}."
+                log.critical(message)
+                raise Exception(message)
             # Test if cache is large enough to hold the new file
             if self.cache_size > 0 and key.size > self.cache_size:
-                log.critical("File %s is larger (%s) than the cache size (%s). Cannot download.",
-                             rel_path, key.size, self.cache_size)
+                log.critical(
+                    "File %s is larger (%s) than the cache size (%s). Cannot download.",
+                    rel_path,
+                    key.size,
+                    self.cache_size,
+                )
                 return False
             if self.use_axel:
                 log.debug("Parallel pulled key '%s' into cache to %s", rel_path, self._get_cache_path(rel_path))
                 ncores = multiprocessing.cpu_count()
                 url = key.generate_url(7200)
-                ret_code = subprocess.call(['axel', '-a', '-n', str(ncores), url])
+                ret_code = subprocess.call(["axel", "-a", "-n", str(ncores), url])
                 if ret_code == 0:
                     return True
             else:
@@ -467,30 +497,44 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
             if os.path.exists(source_file):
                 key = Key(self._bucket, rel_path)
                 if os.path.getsize(source_file) == 0 and key.exists():
-                    log.debug("Wanted to push file '%s' to S3 key '%s' but its size is 0; skipping.", source_file, rel_path)
+                    log.debug(
+                        "Wanted to push file '%s' to S3 key '%s' but its size is 0; skipping.", source_file, rel_path
+                    )
                     return True
                 if from_string:
                     key.set_contents_from_string(from_string, reduced_redundancy=self.use_rr)
                     log.debug("Pushed data from string '%s' to key '%s'", from_string, rel_path)
                 else:
                     start_time = datetime.now()
-                    log.debug("Pushing cache file '%s' of size %s bytes to key '%s'", source_file, os.path.getsize(source_file), rel_path)
+                    log.debug(
+                        "Pushing cache file '%s' of size %s bytes to key '%s'",
+                        source_file,
+                        os.path.getsize(source_file),
+                        rel_path,
+                    )
                     mb_size = os.path.getsize(source_file) / 1e6
                     if mb_size < 10 or (not self.multipart):
                         self.transfer_progress = 0  # Reset transfer progress counter
-                        key.set_contents_from_filename(source_file,
-                                                       reduced_redundancy=self.use_rr,
-                                                       cb=self._transfer_cb,
-                                                       num_cb=10)
+                        key.set_contents_from_filename(
+                            source_file, reduced_redundancy=self.use_rr, cb=self._transfer_cb, num_cb=10
+                        )
                     else:
                         multipart_upload(self.s3server, self._bucket, key.name, source_file, mb_size)
                     end_time = datetime.now()
-                    log.debug("Pushed cache file '%s' to key '%s' (%s bytes transfered in %s sec)",
-                              source_file, rel_path, os.path.getsize(source_file), end_time - start_time)
+                    log.debug(
+                        "Pushed cache file '%s' to key '%s' (%s bytes transfered in %s sec)",
+                        source_file,
+                        rel_path,
+                        os.path.getsize(source_file),
+                        end_time - start_time,
+                    )
                 return True
             else:
-                log.error("Tried updating key '%s' from source file '%s', but source file does not exist.",
-                          rel_path, source_file)
+                log.error(
+                    "Tried updating key '%s' from source file '%s', but source file does not exist.",
+                    rel_path,
+                    source_file,
+                )
         except S3ResponseError:
             log.exception("Trouble pushing S3 key '%s' from file '%s'", rel_path, source_file)
             raise
@@ -506,8 +550,12 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
         if self._in_cache(rel_path):
             if os.path.getsize(self._get_cache_path(rel_path)) == self._get_size_in_s3(rel_path):
                 return True
-            log.debug("Waiting for dataset %s to transfer from OS: %s/%s", rel_path,
-                      os.path.getsize(self._get_cache_path(rel_path)), self._get_size_in_s3(rel_path))
+            log.debug(
+                "Waiting for dataset %s to transfer from OS: %s/%s",
+                rel_path,
+                os.path.getsize(self._get_cache_path(rel_path)),
+                self._get_size_in_s3(rel_path),
+            )
         return False
 
     def _exists(self, obj, **kwargs):
@@ -521,8 +569,8 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
         in_s3 = self._key_exists(rel_path)
         # log.debug("~~~~~~ File '%s' exists in cache: %s; in s3: %s" % (rel_path, in_cache, in_s3))
         # dir_only does not get synced so shortcut the decision
-        dir_only = kwargs.get('dir_only', False)
-        base_dir = kwargs.get('base_dir', None)
+        dir_only = kwargs.get("dir_only", False)
+        base_dir = kwargs.get("base_dir", None)
         if dir_only:
             if in_cache or in_s3:
                 return True
@@ -545,12 +593,11 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
 
     def _create(self, obj, **kwargs):
         if not self._exists(obj, **kwargs):
-
             # Pull out locally used fields
-            extra_dir = kwargs.get('extra_dir', None)
-            extra_dir_at_root = kwargs.get('extra_dir_at_root', False)
-            dir_only = kwargs.get('dir_only', False)
-            alt_name = kwargs.get('alt_name', None)
+            extra_dir = kwargs.get("extra_dir", None)
+            extra_dir_at_root = kwargs.get("extra_dir_at_root", False)
+            dir_only = kwargs.get("dir_only", False)
+            alt_name = kwargs.get("alt_name", None)
 
             # Construct hashed path
             rel_path = os.path.join(*directory_hash_id(self._get_object_id(obj)))
@@ -575,15 +622,14 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
             # If instructed, create the dataset in cache & in S3
             if not dir_only:
                 rel_path = os.path.join(rel_path, alt_name if alt_name else f"dataset_{self._get_object_id(obj)}.dat")
-                open(os.path.join(self.staging_path, rel_path), 'w').close()
-                self._push_to_os(rel_path, from_string='')
+                open(os.path.join(self.staging_path, rel_path), "w").close()
+                self._push_to_os(rel_path, from_string="")
 
     def _empty(self, obj, **kwargs):
         if self._exists(obj, **kwargs):
             return bool(self._size(obj, **kwargs) > 0)
         else:
-            raise ObjectNotFound('objectstore.empty, object does not exist: %s, kwargs: %s'
-                                 % (str(obj), str(kwargs)))
+            raise ObjectNotFound(f"objectstore.empty, object does not exist: {obj}, kwargs: {kwargs}")
 
     def _size(self, obj, **kwargs):
         rel_path = self._construct_path(obj, **kwargs)
@@ -599,10 +645,10 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
 
     def _delete(self, obj, entire_dir=False, **kwargs):
         rel_path = self._construct_path(obj, **kwargs)
-        extra_dir = kwargs.get('extra_dir', None)
-        base_dir = kwargs.get('base_dir', None)
-        dir_only = kwargs.get('dir_only', False)
-        obj_dir = kwargs.get('obj_dir', False)
+        extra_dir = kwargs.get("extra_dir", None)
+        base_dir = kwargs.get("base_dir", None)
+        dir_only = kwargs.get("dir_only", False)
+        obj_dir = kwargs.get("obj_dir", False)
         try:
             # Remove temparory data in JOB_WORK directory
             if base_dir and dir_only and obj_dir:
@@ -632,7 +678,7 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
         except S3ResponseError:
             log.exception("Could not delete key '%s' from S3", rel_path)
         except OSError:
-            log.exception('%s delete error', self._get_filename(obj, **kwargs))
+            log.exception("%s delete error", self._get_filename(obj, **kwargs))
         return False
 
     def _get_data(self, obj, start=0, count=-1, **kwargs):
@@ -648,9 +694,9 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
         return content
 
     def _get_filename(self, obj, **kwargs):
-        base_dir = kwargs.get('base_dir', None)
-        dir_only = kwargs.get('dir_only', False)
-        obj_dir = kwargs.get('obj_dir', False)
+        base_dir = kwargs.get("base_dir", None)
+        dir_only = kwargs.get("dir_only", False)
+        obj_dir = kwargs.get("obj_dir", False)
         rel_path = self._construct_path(obj, **kwargs)
 
         # for JOB_WORK directory
@@ -680,8 +726,7 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
         # even if it does not exist.
         # if dir_only:
         #     return cache_path
-        raise ObjectNotFound('objectstore.get_filename, no cache_path: %s, kwargs: %s'
-                             % (str(obj), str(kwargs)))
+        raise ObjectNotFound(f"objectstore.get_filename, no cache_path: {obj}, kwargs: {kwargs}")
         # return cache_path # Until the upload tool does not explicitly create the dataset, return expected path
 
     def _update_from_file(self, obj, file_name=None, create=False, **kwargs):
@@ -706,8 +751,7 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
             # Update the file on S3
             self._push_to_os(rel_path, source_file)
         else:
-            raise ObjectNotFound('objectstore.update_from_file, object does not exist: %s, kwargs: %s'
-                                 % (str(obj), str(kwargs)))
+            raise ObjectNotFound(f"objectstore.update_from_file, object does not exist: {obj}, kwargs: {kwargs}")
 
     def _get_object_url(self, obj, **kwargs):
         if self._exists(obj, **kwargs):
@@ -724,7 +768,7 @@ class S3ObjectStore(ConcreteObjectStore, CloudConfigMixin):
 
     def shutdown(self):
         self.running = False
-        thread = getattr(self, 'cache_monitor_thread', None)
+        thread = getattr(self, "cache_monitor_thread", None)
         if thread:
             log.debug("Shutting down thread")
             self.sleeper.wake()
@@ -737,14 +781,17 @@ class SwiftObjectStore(S3ObjectStore):
     cache exists that is used as an intermediate location for files between
     Galaxy and Swift.
     """
-    store_type = 'swift'
+
+    store_type = "swift"
 
     def _configure_connection(self):
         log.debug("Configuring Swift Connection")
-        self.conn = boto.connect_s3(aws_access_key_id=self.access_key,
-                                    aws_secret_access_key=self.secret_key,
-                                    is_secure=self.is_secure,
-                                    host=self.host,
-                                    port=self.port,
-                                    calling_format=boto.s3.connection.OrdinaryCallingFormat(),
-                                    path=self.conn_path)
+        self.conn = boto.connect_s3(
+            aws_access_key_id=self.access_key,
+            aws_secret_access_key=self.secret_key,
+            is_secure=self.is_secure,
+            host=self.host,
+            port=self.port,
+            calling_format=boto.s3.connection.OrdinaryCallingFormat(),
+            path=self.conn_path,
+        )
